@@ -54,7 +54,7 @@ use Net::SIP::Util ':all';
 #   @args: either single \%args (hash-ref) or %args (hash) with at least
 #     values for from and to
 #     callid,cseq will be generated if not given
-#     routes will default to [] and usually set from record-route header
+#     routes will default to undef and usually set from record-route header
 #     in response packets
 # Returns: $self
 ############################################################################
@@ -65,7 +65,6 @@ sub new {
 	%$self = %args;
 	$self->{callid} ||= md5_hex( time(), rand(2**32) );
 	$self->{cseq} ||= 0;
-	$self->{route} ||= [];
 	$self->{_transactions} = [];
 	$self->{_cseq_incoming} = 0;
 
@@ -159,6 +158,7 @@ sub new_request {
 		# already a request object
 		$request = $method;
 		$method = $request->method;
+
 	} else {
 
 		# increase cseq unless its explicitly specified
@@ -169,7 +169,7 @@ sub new_request {
 
 		$method = uc($method);
 		my $uri = delete $args{uri};
-		my ($to,$from) = $self->{incoming} ? ($self->{from},$self->{to}) 
+		my ($to,$from) = $self->{incoming} ? ($self->{from},$self->{to})
 			: ($self->{to},$self->{from});
 		if ( !$uri ) {
 			($uri) = sip_hdrval2parts( to => $self->{remote_contact}||$to);
@@ -187,16 +187,18 @@ sub new_request {
 				from => $from,
 				to => $to,
 				$self->{contact} ? ( contact => $self->{contact} ):(),
-				route => $self->{route},
 				cseq => "$cseq $method",
 				'call-id' => $self->{callid},
 				'max-forwards' => 70,
-				%args
+				%args,
 			},
 			$body
-		)
+		);
 	}
 
+	# overwrite any route header in request if we already learned a route
+	$request->set_header( route => $self->{route} ) if $self->{route};
+
 	# create new transaction
 	my %trans = (
 		tid      => $request->tid,
@@ -234,12 +236,14 @@ sub set_callback {
 sub request_delivery_done {
 	my Net::SIP::Endpoint::Context $self = shift;
 	my ($endpoint,$tid,$error) = @_;
+	return if ! $error; # notify of success once I get response
+
 	my $trans = $self->{_transactions};
 	my @ntrans;
 	foreach my $tr (@$trans) {
 		if ( $tr->{tid} eq $tid ) {
 			$self->{_transactions} = \@ntrans;
-			if ( $error && ( my $cb = $tr->{callback} )) {
+			if ( my $cb = $tr->{callback} ) {
 				# permanently failed
 				invoke_callback( $cb, $self,$endpoint,$error );
 			}
@@ -361,17 +365,18 @@ sub handle_response {
 				$req->set_uri( $contact );
 			}
 
+			# use to-tag from this request to update 'to'
+			# FIXME: this should probably be better done by the upper layer
+			# which decides, which call to accept (in case of call-forking with
+			# multiple 2xx responses)
+			$self->{to} = $response->get_header( 'to' ) if ! $self->{incoming};
+
 			# create ACK
 			# if 2xx response changed contact use it as the new URI
 			my $ack = $req->create_ack( $response );
 			invoke_callback($cb,@arg,0,$code,$response,$leg,$from,$ack);
 			$endpoint->new_request( $ack,$self,undef,undef,leg => $leg, dst_addr => $from );
 
-			# use to-tag from this request to update 'to'
-			# FIXME: this should probably be better done by the upper layer
-			# which decides, which call to accept (in case of call-forking with
-			# multiple 2xx responses)
-			$self->{to} = $response->get_header( 'to' ) if ! $self->{incoming};
 
 		} else {
 			# response to ACK, REGISTER...
@@ -414,7 +419,7 @@ sub handle_response {
 	} elsif ( $code == 305 ) {
 		# 21.3.4 305 use proxy
 		# set proxy as the first route and insert request again
-		my $route = $self->{route};
+		my $route = $self->{route} ||= [];
 		unshift @$route,$response->get_header( 'contact' );
 		( my $r = $tr->{request} )->set_header( route => $route );
 		$r->set_cseq( ++$self->{cseq} );
@@ -486,7 +491,9 @@ sub handle_request {
 	my @arg = ($endpoint,$self);
 
 	# extract route information for future requests to the UAC (re-invites)
-	if ( my @route = $request->get_header( 'record-route' )) {
+	# only for INVITE (rfc3261,12.1.1)
+	if ( $method eq 'INVITE' and 
+		my @route = $request->get_header( 'record-route' )) {
 		$self->{route} = \@route;
 	}
 
